<?php

/**
 * Plugin Name: MAP Payment
 * Plugin URI: http://mapisacard.com
 * Description: Платежи при помощи платежного шлюза MAP
 * Version: 0.1
 * Author: MAP
 * Author URI: http://mapisacard.com
 *
 * License: GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once 'api/MapPaymentAPI.php';

add_action( 'plugins_loaded', 'woocommerce_map' );
function woocommerce_map() {

	class WC_Map extends WC_Payment_Gateway {

		private $api;
		public  $testmode;
		/** @var WC_Logger Logger instance */
		public static $logger;

		public function __construct() {
			self::$logger = wc_get_logger();

			$plugin_dir = plugin_dir_url( __FILE__ );

			// Required variables
			$this->id = 'map';
			$this->icon = apply_filters( 'woocommerce_map_icon', '' . $plugin_dir . 'assets/img/map.png' );
			$this->has_fields = false;
			$this->method_title = 'MAP';
			$this->method_description = __( 'Настройки для приема платежей через платежный сервис MAP.' );

			// Load setting field
			$this->init_form_fields();
			$this->init_settings();

			$this->title = $this->get_option( 'title' );
			$this->testmode = 'yes' === $this->get_option( 'testmode', 'no' );

			$this->api = new MapPaymentAPI(
				$this->get_option( 'gateway' ),
				$this->get_option( 'key' ),
				$this->get_option( 'password' ),
				$this->testmode
			);
			$this->api->setRequestMethod( array( get_class( $this ), 'map_send_request' ) );


			add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );

			// Save options
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// MAP notification listener
			add_action( 'woocommerce_api_' . strtolower( get_class( $this ) ), array( $this, 'check_response' ) );
		}

		public static function map_send_request( $url, array $data ) {
			$response = wp_safe_remote_post( $url, array(
				'body' => http_build_query( $data )
			) );

			if ( is_wp_error( $response ) ) {
				self::$logger->log( 'error', 'Ошибка запроса:' . $response->get_error_message() );
				throw new Exception( 'Ошибка запроса' );
			}

			if ( empty( $response['body'] ) ) {
				self::$logger->log( 'error', 'Ответ на запрос отсутсвует. Данные запроса: ' . wc_print_r( $data, true ) );
				throw new Exception( 'Ответ на запрос отсутсвует' );
			}

			return json_decode( $response['body'], true );
		}

		public function init_form_fields() {
			$this->form_fields = array(
				'enabled'             => array(
					'title'   => __( 'Состояние способа оплаты' ),
					'type'    => 'checkbox',
					'label'   => __( 'Включен' ),
					'default' => 'yes',
				),
				'testmode'            => array(
					'title'   => __( 'Тестовый режим' ),
					'type'    => 'checkbox',
					'label'   => __( 'Включить отправку данных на тестовый сервер' ),
					'default' => 'no',
				),
				'title'               => array(
					'title'       => __( 'Название способа оплаты' ),
					'type'        => 'text',
					'description' => __( 'Отображается пользователю при оформлении заказа' ),
					'default'     => __( 'Платежный сервис MAP' ),
				),
				'gateway'			  => array(
					'title' 	  => __( 'Адрес платежного шлюза' ),
					'type' 		  => 'text',
					'default'	  => '',
				),
				'key'                 => array(
					'title'       => __( 'Идентификатор продавца' ),
					'type'        => 'text',
					'description' => __( 'Выдается продавцу с параметрами тестового/боевого доступа' ),
					'default'     => '',
				),
				'password'            => array(
					'title'       => __( 'Пароль' ),
					'type'        => 'password',
					'description' => __( 'Пароль терминала для совершения операций' ),
					'default'     => '',
				),
				'order_auto_complete' => array(
					'title'   => __( 'Автозавершение платежа' ),
					'type'    => 'checkbox',
					'label'   => __( 'Переводить статус платежа в "Выполнен" после успешной оплаты' ),
					'default' => 'no',
				),
				'ofd_enable'          => array(
					'title'   => __( 'Передача данных в ОФД' ),
					'type'    => 'checkbox',
					'label'   => __( 'Передавать список товаров и цен согласно ФЗ-54' ),
					'default' => 'no',
				)
			);
		}

		public function receipt_page( $order_id ) {
			$order = wc_get_order( $order_id );
			$session_id = $this->get_session_id( $order );
			$this->generate_payment_form( $session_id );
		}

		public function process_payment( $order_id ) {
			$order = wc_get_order( $order_id );
			$session_id = $this->get_session_id( $order );

			if ( ! empty( $session_id ) ) {
				$payment_state = $this->api->getPaymentStateRequest( $session_id, true );
				if ( $this->api::isSuccessState( $payment_state ) ) {
					$this->validate_amount( $order, $payment_state['Amount'] );
					$this->set_order_paid( $order );
					return array(
						'result'   => 'success',
						'redirect' => $this->get_return_url( $order )
					);
				}
			}

			$data = array(
				'OrderId'      => $order_id,
				'Amount'       => $order->get_total() * 100,
				'Type'         => 'Pay',
				'CustomParams' => array(
					'Backurl' => wc()->api_request_url( 'wc_' . $this->id )
				),
			);

			if ( $this->get_option( 'ofd_enable' ) === 'yes' ) {
				$data['Goods'] = $this->get_goods_string( $order );
				$data['CustomParams']['Email'] = $order->get_billing_email();
			}

			$this->init_session( $order, $data );

			return array(
				'result'   => 'success',
				'redirect' => $order->get_checkout_payment_url( true )
			);
		}

		public function check_response() {
			$order_id = $_GET['order_id'];
			$order = wc_get_order( $order_id );

			if ( $_GET['success'] ) {
				$payment_state = $this->api->getPaymentStateRequest( $_GET['merchant_order_id'] );
				if ( $this->api::isSuccessState( $payment_state ) ) {
					$this->validate_amount( $order, $payment_state['Amount'] );
					$this->set_order_paid( $order, $_GET['merchant_order_id'] );
					wp_redirect( $this->get_return_url( $order ) );
					die();
				}

				// TODO описать код ошибки в развернутом виде (см. документацию по api)
				wp_die( 'Статус платежа не подтвержден: <strong>' . $payment_state['ErrCode'] . '</strong>',
					__( 'Ошибка проверки статуса' ),
					array( 'response' => '200' ) );

			} else {
				$order->add_order_note( 'Заказ не был оплачен' );
				$order->update_status( 'failed' );
				wp_redirect( $order->get_cancel_order_url_raw() );
				die();
			}
		}

		private function init_session( WC_Order $order, array $data ) {
			$request = $this->api->init( $data );
			$session = new MapSession( $request );

			if ( $session->isSuccess() ) {
				$order->update_meta_data( '_map_session_id', $session->getSessionGUID() );
				$order->save();
				return $session->getSessionGUID();
			}

			wc_add_notice( __( 'Ошибка создания сессии:' ) . $session->getErrCode(), 'error' );
			return null;
		}

		private function generate_payment_form( $session_id ) {
			?>
			<form name="map_buy_form" action="<?php echo $this->api->getPaymentActionUrl(); ?>">
				<input name="sessionId" value="<?php echo htmlspecialchars( $session_id ); ?>" type="hidden"/>
				<button type="submit">Оплатить</button>
			</form>
			<?php
		}

		private function validate_amount( WC_Order $order, $amount ) {
			if ( (int) $order->get_total() * 100 !== $amount ) {
				$error_msg = sprintf( 'Ошибка проверки суммы: сумма заказа не совпадает с оплаченной (%s).',
					number_format( $amount / 100, 2, '.', '' ) );
				$order->update_status( 'on-hold', $error_msg );
				wp_die( $error_msg, __( 'Ошибка проверки оплаты' ) );
			}
		}

		private function get_session_id( WC_Order $order ) {
			return $order->get_meta( '_map_session_id' );
		}

		private function get_product_top_level_category( $product_id ) {
			$product_terms = get_the_terms( $product_id, 'product_cat' );
			$product_category = $product_terms[0];
			$product_category_parent = $product_category->parent;
			$product_top_category = $product_category->term_id;

			while ( $product_category_parent !== 0 ) {
				$product_category_term = get_term( $product_category_parent, 'product_cat' );
				$product_category_parent = $product_category_term->parent;
				$product_top_category = $product_category_term->term_id;
			}

			return $product_top_category;
		}

		private function get_goods_string( WC_Order $order ) {
			$items = array();

			/** @var WC_Order_Item_Product $item */
			foreach ( $order->get_items() as $item_id => $item ) {
				/** @var WC_Product $product */
				$product = $item->get_product();
				$first_cat_id = $this->get_product_top_level_category( $product->get_id() );
				$fz_attr = get_term_meta( $first_cat_id, 'fz_attr', true );
				if ( empty( $fz_attr ) ) {
					$fz_attr = 1;
				}

				$count = $item->get_quantity();
				while ( $count-- ) {
					$items[] = $item->get_name() . '=' . $order->get_item_total( $item, false, false ) * 100 . ',' . $fz_attr;
				}
			}

			if ( wc()->cart->needs_shipping() ) {
				$items[] = __( 'Доставка' ) . '=' . $order->get_shipping_total() * 100 . ',4';  // 4 - услуга
			}

			return implode( ';', $items );
		}

		private function set_order_paid( WC_Order $order, $transaction_id = '' ) {
			if ( $this->get_option( 'order_auto_complete' ) === 'yes' ) {
				wc_reduce_stock_levels( $order->get_id() );
				wc_empty_cart();

				try {
					if ( ! empty( $transaction_id ) ) {
						$order->set_transaction_id( $transaction_id );
					}
					$order->set_date_paid( current_time( 'timestamp', true ) );
				} catch ( WC_Data_Exception $e ) {
					$this::$logger->error( __( 'Оплата заказа завершилась ошибкой' ), array(
						'order' => $order,
						'error' => $e
					) );

					return false;
				}

				$order->update_status( 'completed', 'Платеж успешно оплачен' );
			} else {
				$order->payment_complete( $transaction_id );
			}
		}

	}


	function add_map_gateway( $methods ) {
		$methods[] = 'WC_Map';
		return $methods;
	}

	add_filter( 'woocommerce_payment_gateways', 'add_map_gateway' );

	add_action( 'product_cat_add_form_fields', function () {
		$options = array(
			1 => 'Товар',
			'Продакцизный товар',
			'Работа',
			'Услуга',
			'Ставка азартной игры',
			'Выигрыш азартной игры',
			'Лотерейный билет',
			'Выигрыш лотереи',
			'Предоставление РИД',
			'Платеж',
			'Агентское вознаграждение',
			'Составной предмет расчета',
			'Иной предмет расчета',
		);
		?>
		<div class="form-field">
			<label for="fz_attr">Признак предмета расчета</label>
			<select id="fz_attr" name="fz_attr">
				<?php foreach ( $options as $key => $value ) {
					echo '<option value="' . esc_attr( $key ) . '">' . $value . '</option>';
				}
				?>
			</select>
			<p>
				Признак, необходимый для отправки данных в ОФД согласно 54-ФЗ. Используется при оплате через платежный
				модуль "MAP".
			</p>
		</div>
		<?php
	}, 10, 1 );

	add_action( 'product_cat_edit_form_fields', function ( $term ) {
		$options = array(
			1 => 'Товар',
			'Продакцизный товар',
			'Работа',
			'Услуга',
			'Ставка азартной игры',
			'Выигрыш азартной игры',
			'Лотерейный билет',
			'Выигрыш лотереи',
			'Предоставление РИД',
			'Платеж',
			'Агентское вознаграждение',
			'Составной предмет расчета',
			'Иной предмет расчета',
		);
		$term_id = $term->term_id;
		$fz_term = get_term_meta( $term_id, 'fz_attr', true );
		?>
		<tr class="form-field">
			<th scope="row" valign="top"><label>Признак предмета расчета</label></th>
			<td>
				<select id="fz_attr" name="fz_attr">
					<?php foreach ( $options as $key => $value ) {
						echo '<option value="' . esc_attr( $key ) . '" ' . selected( $key, $fz_term ) . '>' . $value . '</option>';
					}
					?>
				</select>
				<p class="description">
					Признак, необходимый для отправки данных в ОФД согласно 54-ФЗ. Используется при оплате через
					платежный модуль "MAP".
				</p>
			</td>
		</tr>
		<?php
	}, 10, 1 );

	add_action( 'edited_product_cat', 'fz_save_term', 10, 1 );
	add_action( 'create_product_cat', 'fz_save_term', 10, 1 );
	function fz_save_term( $term_id ) {
		$val = filter_input( INPUT_POST, 'fz_attr' );
		update_term_meta( $term_id, 'fz_attr', $val );
	}


}
